unit Demounit;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Btrvcomp, StdCtrls, Btrvcmp3, Mask, BTRVCMP2, Buttons,
  ExtCtrls, wincrt;

type
  TDemoForm = class(TForm)
    BtrvDB1: TBtrvDB;
    TitleEdit: TBtrvEdit;
    Address1Edit: TBtrvEdit;
    Address2Edit: TBtrvEdit;
    CityEdit: TBtrvEdit;
    StateEdit: TBtrvEdit;
    CountryEdit: TBtrvEdit;
    DOBMaskEdit: TBtrvMaskEdit;
    PhoneMaskEdit: TBtrvMaskEdit;
    MailingCodeMaskEdit: TBtrvMaskEdit;
    BtrvFile1: TBtrvFile;
    TitleLabel: TLabel;
    DOBLabel: TLabel;
    Address1Label: TLabel;
    Address2Label: TLabel;
    CityLabel: TLabel;
    StateLabel: TLabel;
    CountryLabel: TLabel;
    MailingCodeLabel: TLabel;
    PhoneLabel: TLabel;
    ContactDriverCombo: TBtrvDriverComboBox;
    ContactLabel: TLabel;
    ComponentInfoButton: TBitBtn;
    Image1: TImage;
    SortingPanel: TPanel;
    SortByLabel: TLabel;
    LastNameRB: TRadioButton;
    FullNameRB: TRadioButton;
    DOBRB: TRadioButton;
    StateRB: TRadioButton;
    CountryRB: TRadioButton;
    StateCityRB: TRadioButton;
    CountryStateCityRB: TRadioButton;
    procedure ParseContactName(var LastName: String;
                               var FirstName: String;
                               var MiddleInitial: String);
    procedure SortChange(Sender: TObject);
    procedure ProcessContactFormat;
    procedure ComponentInfoButtonClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ContactDriverComboKeyPress(Sender: TObject; var Key: Char);
    procedure ContactDriverComboEnter(Sender: TObject);
    procedure ContactDriverComboExit(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ContactDriverComboClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  DemoForm: TDemoForm;

implementation

{$R *.DFM}

procedure TDemoForm.SortChange(Sender: TObject);
begin
  {Set the new key}
  BtrvFile1.KeyNumber := (Sender AS TComponent).Tag;

  {Clear the DriverComboBox}
  ContactDriverCombo.Clear;

  {Repopulate the ContactDriverCombo with the new order based on the new key}
  ContactDriverCombo.Populate;
end;

procedure TDemoForm.ComponentInfoButtonClick(Sender: TObject);
var
  HelpFileName: array[0..40] of char;
  Counter: Integer;
  SearchLocations, FileLocation: String;
const
  ContentsContextNumber: Integer = 10000;

begin
  {Always watch your back}
  inherited Click;

  {Where to look}  {Yes, ugly and crude, but direct and functional.}
  SearchLocations := 'C:\DELPHI\BIN;D:\DELPHI\BIN\;' +
                   'E:\DELPHI\BIN\;F:\DELPHI\BIN;G:\DELPHI\BIN;H:\DELPHI\BIN;' +
                   'I:\DELPHI\BIN;J:\DELPHI\BIN;K:\DELPHI\BIN;L:\DELPHI\BIN;' +
                   'M:\DELPHI\BIN;N:\DELPHI\BIN;O:\DELPHI\BIN;P:\DELPHI\BIN;' +
                   'Q:\DELPHI\BIN;R:\DELPHI\BIN;S:\DELPHI\BIN;T:\DELPHI\BIN;' +
                   'U:\DELPHI\BIN;V:\DELPHI\BIN;W:\DELPHI\BIN;X:\DELPHI\BIN;' +
                     'Y:\DELPHI\BIN;Z:\DELPHI\BIN;';

  {Hunt the Help file.}
  FileLocation := FileSearch('BTRVCOMP.HLP',SearchLocations);

  {If we did not find it, let the user know and step out.}
  If FileLocation = '' then
    Begin
      MessageDlg('The BTRVCOMP.HLP file was not found in \DELPHI\BIN\',
               mtError, [mbOk], 0);
      Exit;
    End;

  {null terminated is important}
  For Counter := 0 to 40 do
    HelpFileName[Counter] := #0;

  {Set up the name}
  StrPCopy(HelpFileName,FileLocation);

  {Call up the Contents Page of the BTRVCOMP.HLP file}
  WinHelp(Handle, HelpFileName, HELP_CONTEXT, ContentsContextNumber);

end;

procedure TDemoForm.FormActivate(Sender: TObject);
var
  ReturnInteger:                           Integer;
  MsgString, EXEDirectory, LookingFor:     String;
  NewFileDef:                              TFileElement;
const
  DataFile: String = 'DEMOFILE.SNG';

begin
  {If our datafile exists in the directory the EXE is running from, OK}
  {Otherwise we have to create it}

  NewFileDef := nil;

  {The current directory}
  EXEDirectory := ExtractFilePath(Application.ExeName);

  {The datafile should be here}
  LookingFor := ExeDirectory + DataFile;

  {If it is not here then create the datafile here.}
  If NOT FileExists(LookingFor) Then

    {If it does not exist, then you cannot use it!  -AHS}
    If BtrvDB1.Repository <> nil Then
      Begin
        {Find the FileElement of the Repository that we are going to create}
        NewFileDef := BtrvDB1.Repository.FileElement('DEMO');

        {If we did not find the DEMO alias, then this is an error}
        If NewFileDef = nil Then
          Begin
            MessageDlg('DEMO is not in the current Repository',
                         mtError, [mbOk], 0);

            {Kill the App}
            Close;
          End;

        {Edit Repository File Alias to match where the EXE is now}
        NewFileDef.FileName := LookingFor;

        {Create the new file}
        ReturnInteger := BtrvDB1.CreateFileFromRepository('DEMO');

        {Test the return for success}
        If ReturnInteger <> 0 then
          Begin
            MsgString := 'An Error occured while trying to create ' +
                      LookingFor + '.'#13 +
                      'Please remedy this before trying to run the demo again.';

            MessageDlg(MsgString, mtError, [mbOk], 0);

            {Kill the App}
            Close;
          end
      end
    else
      Begin
        MessageDlg('A Repository does not exist for this application', mtError,
                   [mbOk], 0);

       {Kill the App}
        Close;
      End;

  {Everything is in place...}
  {  Make sure the Repository points at the datafile}
  BtrvDB1.Repository.FileElement('DEMO').FileName := LookingFor;

  {Refresh the path in BtrvFile1}
  BtrvFile1.Alias := 'DEMO';

  {Open the file (it had its AutoLoad set to false on create)}
  ReturnInteger := BtrvFile1.Open(BtrvFile1.BtrvOwner,BtrvFile1.AccessMode);

  {Verify the Open succeeded.
   This could be done by leaving BtrvDB.ExceptionsOn = True and
   catching the exception, but I chose to test it myself here.
   This worked everytime I tested it, so I have reasonable confidence in it.}
  If ReturnInteger <> 0 Then
    Begin
      MessageDlg('I was unable to Open the file: ' + LookingFor, mtError,
                  [mbOk], 0);

      {Close the app}
      Close;
    End;

end;

procedure TDemoForm.ContactDriverComboKeyPress(Sender: TObject;
  var Key: Char);
begin
  {If the Enter key is used then}
  If Key = #13 Then
    Begin
      {process the format of the entry, utilizing the Btrv inherited Exit
        to populate the list with the new item (if it is new)}
      ContactDriverComboExit(Self);

      {Set focus to the next field}
      PhoneMaskEdit.SetFocus;
    end;
end;

procedure TDemoForm.ParseContactName(var LastName: String;
                                     var FirstName: String;
                                     var MiddleInitial: String);
var
  TestString: String;
  EndLocation, StartLocation, CurrentLocation: Integer;
Begin
  {Verify that the entry in the Contact Combo Box has:
       LastName no longer than 35,
       FirstName no longer than 20,       OPTIONAL
       Middle Initial no longer than 1    OPTIONAL}
  TestString := ContactDriverCombo.Text;

  {Prepare for Parsing}
  EndLocation := Length(TestString);
  StartLocation := 0;
  CurrentLocation := 0;
  LastName := '';
  FirstName := '';
  MiddleInitial := '';

  {Find out where the last name stops}
  While CurrentLocation <= EndLocation do
    If (Copy(TestString,CurrentLocation,1) = ',') OR
       (Copy(TestString,CurrentLocation,1) = ' ') Then
      Break
    else
      Inc(CurrentLocation);

  {Set LastName to the Last Name}
  LastName := Copy(TestString,StartLocation,CurrentLocation - 1);

  {If a comma is next then skip it}
  If Copy(TestString,CurrentLocation,1) = ',' Then
    Inc(CurrentLocation);

  {If a space is next then skip it}
  If Copy(TestString,CurrentLocation,1) = ' ' Then
    Inc(CurrentLocation);

  {Synch Start with Current name for the First Name}
  StartLocation := CurrentLocation;

  {Find out where the first name ends}
  While CurrentLocation <= EndLocation do
    If Copy(TestString,CurrentLocation,1) = ' ' Then
      Break
    else
      Inc(CurrentLocation);

  {Set First Name to the First Name}
  FirstName := Copy(TestString,StartLocation,CurrentLocation - StartLocation);

  {See if a middle initial was given}
  While (Copy(TestString,CurrentLocation,1) = ' ') AND
        (CurrentLocation <= EndLocation) Do
       Inc(CurrentLocation);

  {I'm possibly at the middle initial, so test and act accordingly}
  If Copy(TestString,CurrentLocation,1) <> ' ' Then
    MiddleInitial := Copy(TestString,CurrentLocation,1);
End;

procedure TDemoForm.ProcessContactFormat;
var
  TestString, FirstName, LastName, Blanks,
  MiddleInitial, ContactName:             String;
  ReturnInteger:                         Integer;
  ContactField:                         TBtrvField;
Begin
  Blanks := '                                           ';
  {Verify the entry is now in the ComboBox,
    If it is not it was not successfully inserted into the database.}
  If ContactDriverCombo.Items.IndexOf(ContactDriverCombo.Text) < 0 Then
      Begin
        MessageDlg('The record was not successfully inserted into DEMOFILE.SNG',
                   mtError, [mbOk], 0);
        Exit;
      end;

  {Get the names to use}
  ParseContactName(LastName, FirstName, MiddleInitial);

  {Construct the new contactname (for our "unusual" search)}
  ContactName := LastName + Copy(Blanks,0,35 - Length(LastName)) +
                 FirstName + Copy(Blanks,0,20 - Length(FirstName)) +
                 MiddleInitial + Copy(Blanks,0,1 - Length(MiddleInitial));

  {LastName, FirstName, MiddleInitial are now set}
  ContactDriverCombo.Text := LastName + ' ' + FirstName + ' ' + MiddleInitial;

  {We must update the record with correct parsing}

  {Start out somewhere}
  ContactField := nil;

  {Create the fields to move the First, Last, Middles into the buffer}
  MakeEditField(ContactField, dbString, 0, 56, 0);

  {Insert the new Last and First Names and MiddleInitial}
  ContactField.TextToData(ContactName);

  BtrvFile1.RecordBuffer.WriteField(ContactField);

  {Insert the new record}
  ReturnInteger := BtrvFile1.Update(KeepPosition);

  {Notify if the Update had a problem}
  If ReturnInteger <> 0 then
    MessageDlg('The Update Failed. The Status Code was ' +
               IntToStr(ReturnInteger), mtError, [mbOk], 0)
  Else
    {Clear the phone number}
     PhoneMaskEdit.Clear;

  {Show the handiwork}
  BtrvFile1.UpdateComponents;
End;

procedure TDemoForm.ContactDriverComboEnter(Sender: TObject);
var
  ReturnInteger: Integer;
begin
  {If nothing is in the list, do nothing}
  If ContactDriverCombo.Items.Count = 0 Then
    Exit;

  {If something is in the text field and the buffer is dirty then
     update the record.}
  If (ContactDriverCombo.Text <> '') Then
    Begin
      If (BtrvFile1.RecordBuffer.Dirty) Then
        Begin
          ReturnInteger := BtrvFile1.Update(KeepPosition);

          {Notify if the update had a problem}
          If ReturnInteger <> 0 then
            MessageDlg('The Update Failed. The Status Code was: ' +
                       IntToStr(ReturnInteger), mtError, [mbOk], 0);
        End;
    End
  Else
    PhoneMaskEdit.Clear;
end;

procedure TDemoForm.ContactDriverComboExit(Sender: TObject);
begin
  If ContactDriverCombo.Text <> '' Then
    inherited Click;

  {Only want to do this on a new record}
  If StateEdit.Text = '' Then
    {Update the File fields correctly}
    ProcessContactFormat;

end;

procedure TDemoForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  StatusCode: Integer;
begin
  {Everyone was taught to clean up after themselves...}
  StatusCode := BtrvFile1.Close;

  {But sometime the mess is outside of their resources.}
  If StatusCode > 0 then
    MessageDlg('I was unable to Close DEMOFILE.SNG.'#13 +
               'The Status Code was ' + IntToStr(StatusCode),
               mtError, [mbOk], 0)
  Else
    if StatusCode < 0 Then
      MessageDlg('I was unable to Close DEMOFILE.SNG.'#13 +
               'I do not believe it was Open.'#13 +
               'The Status Code was ' + IntToStr(StatusCode),
               mtError, [mbOk], 0);

  {Standard Delphi}
  Action := caFree;
end;

procedure TDemoForm.ContactDriverComboClick(Sender: TObject);
var
  TempString, Blanks, LastName, FirstName, MiddleInitial: String;
  TempField: TBtrvField;
begin
  Blanks := '                                      ';

  {When a name is selected, we'll find the correct record here.}
  {A driver will do this in simple cases, but we are doing some
   different stuff here:
     Lastname, firstname, and middle initial are stored not as one
     name but as seperate pieces in the record.
     The driver looks for a match on Btrv1* or the first alias,
     and there could be two matches on last name.}

   {Make sure we are working with something that exists.}
   If ContactDriverCombo.ItemIndex < 0 then Exit;

   {Make sure we are working with the correct object}
   If (ContactDriverCombo.Items.Objects[ContactDriverCombo.ItemIndex]
      Is TBtrvField) Then
     Begin
       {Get our Object}
       TempField :=
       (ContactDriverCombo.Items.Objects[ContactDriverCombo.ItemIndex]
        AS TBtrvField);

       {Kill it and remake it as we want it}
       MakeEditField(TempField, dbString, 0, 56, 0);

       {Raw data to go to file}
       ParseContactName(LastName, FirstName, MiddleInitial);

       {Format it like we want to see it}
       TempString := LastName + Copy(Blanks,0,35 - Length(LastName)) +
                     FirstName + Copy(Blanks,0,20 - Length(FirstName)) +
                     MiddleInitial + Copy(Blanks,0,1 - Length(MiddleInitial));

       {Put it in the object}
       TempField.TextToData(TempString);

       {MakeEditField destroys then creates, so I have to add the new object}
       ContactDriverCombo.Items.Objects[ContactDriverCombo.ItemIndex] := TempField;

   {This "tricks" the DriverComponent to picking the record that matches
    the field (there ought to be only one record)
    The Default Handling will do the rest for me.}
     End;
end;

end.
